/**
 * @copyright 2021 Double Bastion LLC <www.doublebastion.com>
 *
 * @author Double Bastion LLC
 *
 * @license GNU AGPL version 3 or any later version
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU AFFERO GENERAL PUBLIC LICENSE for more details.
 *
 * You should have received a copy of the GNU Affero General Public
 * License along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

$(document).ready(function() {

   var baseUrl = OC.generateUrl('/apps/sip_trip_phone/user/getsettings');
   var userid = "<?php p($userId); ?>";

   $.ajax({
      method: 'GET',
      url: baseUrl + '/' + userid,
      contentType: 'application/json',
      success: function(settingsdb) {

         var connectData = {
             "User" : settingsdb.sipusername,
             "Realm" : settingsdb.siprealm,
             "Display" : settingsdb.pdisplayname,
             "WSServer" : settingsdb.stphwssurl,
             "Stun" : settingsdb.stunserver
         };

         localStorage.setItem('SIPCreds', JSON.stringify(connectData));

	 var baseUrl = OC.generateUrl('/apps/sip_trip_phone/user/getsippass');

         $.ajax({
	      method: 'GET',
	      url: baseUrl + '/' + userid,
              contentType: 'text',
	      success: function(sippassword) {

                 var url = OC.generateUrl('/apps/sip_trip_phone/phone/index.html');

		 var features = 'menubar=no,location=no,resizable=no,scrollbars=no,status=no,addressbar=no,width=320,height=490';

                 if (!localStorage.getItem('SipTripPhone')) {
		     var newPopUpWindow = window.open(url, 'SipTripPhone', features);
                     window.sipUserPasswd = sippassword;
		     return false;
		 } else {
		     window.alert('The phone is already open !');
		 }
              }
         });

      }
   });
});

